/**
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Reister ( fabian dot reister at kit dot edu )
 * @date       2023
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

#include <string>
#include <tuple>
#include <vector>

#include <Eigen/Core>

namespace armarx::navigation::algorithms
{

    struct Room
    {
        std::string name;

        std::vector<Eigen::Vector2f> polygon;
        float height;

        float zFloor = 0;

        bool isInside(const Eigen::Vector3f& point, bool restrictTo2dCheck = false) const;

        Eigen::Vector2f center() const;

        std::tuple<Eigen::Vector2f, Eigen::Vector2f> aabb() const;
        bool isInside(const Eigen::Vector2f& point) const;
    };

} // namespace armarx::navigation::algorithms
