#pragma once

#include <functional>

#include <armarx/navigation/core/events.h>
#include <armarx/navigation/trajectory_control/local/TrajectoryController.h>

namespace armarx::navigation::client
{

    using GlobalTrajectoryUpdatedCallback =
        std::function<void(const core::GlobalTrajectoryUpdatedEvent&)>;
    using LocalTrajectoryUpdatedCallback =
        std::function<void(const core::LocalTrajectoryUpdatedEvent&)>;
    using TrajectoryControllerUpdatedCallback =
        std::function<void(const traj_ctrl::local::TrajectoryControllerResult&)>;
    using GlobalPlanningFailedCallback =
        std::function<void(const core::GlobalPlanningFailedEvent&)>;

    using LocalPlanningFailedCallback = std::function<void(const core::LocalPlanningFailedEvent&)>;

    using OnMovementStartedCallback = std::function<void(const core::MovementStartedEvent&)>;

    using OnGoalReachedCallback = std::function<void(const core::GoalReachedEvent&)>;
    using OnWaypointReachedCallback = std::function<void(const core::WaypointReachedEvent&)>;
    using OnSafetyThrottlingTriggeredCallback =
        std::function<void(const core::SafetyThrottlingTriggeredEvent&)>;
    using OnSafetyStopTriggeredCallback =
        std::function<void(const core::SafetyStopTriggeredEvent&)>;
    using OnUserAbortTriggeredCallback = std::function<void(const core::UserAbortTriggeredEvent&)>;
    using OnInternalErrorCallback = std::function<void(const core::InternalErrorEvent&)>;

    class EventSubscriptionInterface
    {

    public:
        virtual void onMovementStarted(const OnMovementStartedCallback& callback) = 0;
        virtual void onGoalReached(const OnGoalReachedCallback& callback) = 0;

        virtual void onWaypointReached(const OnWaypointReachedCallback& callback) = 0;

        virtual void
        onSafetyThrottlingTriggered(const OnSafetyThrottlingTriggeredCallback& callback) = 0;

        virtual void onSafetyStopTriggered(const OnSafetyStopTriggeredCallback& callback) = 0;

        virtual void onUserAbortTriggered(const OnUserAbortTriggeredCallback& callback) = 0;

        virtual void onInternalError(const OnInternalErrorCallback& callback) = 0;

        virtual void onGlobalPlanningFailed(const GlobalPlanningFailedCallback& callback) = 0;
        virtual void onLocalPlanningFailed(const LocalPlanningFailedCallback& callback) = 0;

        virtual void onGlobalTrajectoryUpdated(const GlobalTrajectoryUpdatedCallback& callback) = 0;
        virtual void onLocalTrajectoryUpdated(const LocalTrajectoryUpdatedCallback& callback) = 0;


        // Non-API.
        virtual ~EventSubscriptionInterface() = default;
    };

} // namespace armarx::navigation::client
