#pragma once


#include <mutex>
#include <string>
#include <vector>

#include <ArmarXCore/core/services/tasks/PeriodicTask.h>

#include <RobotAPI/libraries/armem/client/Reader.h>
#include <RobotAPI/libraries/armem/core/MemoryID.h>
#include <RobotAPI/libraries/armem/core/forward_declarations.h>
#include <RobotAPI/libraries/armem/core/wm/memory_definitions.h>

#include <armarx/navigation/client/services/SimpleEventHandler.h>

namespace armarx::armem::client
{
    class MemoryNameSystem;
}

namespace armarx::navigation::client
{

    class MemoryPolling : virtual public SimpleEventHandler
    {

    public:
        MemoryPolling(const std::string& callerId, armem::client::MemoryNameSystem& mns);

        void handleEvent(const armem::wm::EntityInstance& memoryEntity);

        // Non-API.
    public:
        ~MemoryPolling() override;


    protected:
        void runPollMemoryEvents();
        void handleEvents(const armem::wm::Memory& memory);


        void onEntityUpdate(const std::vector<armem::MemoryID>& snapshotIDs);

    private:
        const std::string callerId;
        armem::client::MemoryNameSystem& memoryNameSystem;

        armem::Time lastMemoryEventReceived;

        armem::client::Reader memoryReader;

        std::mutex eventHandlingMtx;

        armarx::PeriodicTask<MemoryPolling>::pointer_type task;
    };

} // namespace armarx::navigation::client
