/**
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    armarx::navigation::components::costmap_merger
 * @author     Tobias Gröger ( tobias dot groeger at student dot kit dot edu )
 * @date       2025
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */


#pragma once

#include <mutex>
#include <string>
#include <vector>

#include <ArmarXCore/core/Component.h>
#include <ArmarXCore/core/application/properties/PropertyDefinitionContainer.h>

#include <RobotAPI/libraries/armem/client/plugins/ListeningPluginUser.h>
#include <RobotAPI/libraries/armem/client/plugins/ReaderWriterPlugin.h>
#include <RobotAPI/libraries/armem/core/MemoryID.h>

#include <armarx/navigation/components/costmap_merger/ComponentInterface.h>
#include <armarx/navigation/memory/client/costmap/Reader.h>
#include <armarx/navigation/memory/client/costmap/Writer.h>

namespace armarx::navigation::components::costmap_merger
{

    class Component :
        virtual public armarx::Component,
        virtual public armarx::navigation::components::costmap_merger::ComponentInterface,
        virtual public armarx::armem::ListeningClientPluginUser
    {
    public:
        Component();

        /// @see armarx::ManagedIceObject::getDefaultName()
        std::string getDefaultName() const override;

        /// Get the component's default name.
        static std::string GetDefaultName();


    protected:
        /// @see PropertyUser::createPropertyDefinitions()
        armarx::PropertyDefinitionsPtr createPropertyDefinitions() override;

        /// @see armarx::ManagedIceObject::onInitComponent()
        void onInitComponent() override;

        /// @see armarx::ManagedIceObject::onConnectComponent()
        void onConnectComponent() override;

        /// @see armarx::ManagedIceObject::onDisconnectComponent()
        void onDisconnectComponent() override;

        /// @see armarx::ManagedIceObject::onExitComponent()
        void onExitComponent() override;


        void mergeAndStoreCostmap();

        void processObjectInstance(const armem::MemoryID& id,
                                   const std::vector<armem::MemoryID>& snapshotIDs);

    private:
        static const std::string defaultName;

        struct Properties
        {
            std::vector<std::string> costmapsToMerge;
            std::string outputCostmapName = "distance_to_obstacles";
        };

        Properties properties;

        armem::client::plugins::ReaderWriterPlugin<memory::client::costmap::Reader>*
            costmapReaderPlugin = nullptr;

        armem::client::plugins::ReaderWriterPlugin<memory::client::costmap::Writer>*
            costmapWriterPlugin = nullptr;

        std::mutex mergeCostmapMtx;
    };

} // namespace armarx::navigation::components::costmap_merger
