/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Tobias Gröger ( tobias dot groeger at student dot kit dot edu )
 * @date       2021
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

#include <cstddef>
#include <optional>
#include <vector>

#include <armarx/navigation/components/laser_scanner_feature_extraction/ChainApproximation.h>
#include <armarx/navigation/components/laser_scanner_feature_extraction/FeatureExtractor.h>

namespace armarx::navigation::components::laser_scanner_feature_extraction
{

    class FeatureMerger
    {
    public:
        FeatureMerger(std::vector<Features>&& features,
                      const ChainApproximation::Params& chainApproximationParams);
        FeatureMerger(const std::vector<Features>& features,
                      const ChainApproximation::Params& chainApproximationParams);

        std::vector<Features> merge();

    public:
        static bool insideConvexPoly(const Eigen::Vector2f& p,
                                     const std::vector<Eigen::Vector2f>& vertices);

    private:
        std::optional<std::size_t>
        checkPoint(const Eigen::Vector2f p,
                   std::size_t ignore,
                   const std::optional<std::size_t> checkFirst = std::nullopt);
        void addToFeature(Features& f, const Features& toAdd);
        void recalculateFeatures(std::vector<Features>& features);

    private:
        std::vector<Features> inputFeatures;
        const ChainApproximation::Params chainApproximationParams;
    };

} // namespace armarx::navigation::components::laser_scanner_feature_extraction
