/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    RobotComponents::ArmarXObjects::LaserScannerFeatureExtraction
 * @author     Fabian Reister ( fabian dot reister at kit dot edu )
 * @date       2021
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#include <iostream>

#include <ArmarXCore/core/logging/Logging.h>

#include <armarx/navigation/components/laser_scanner_feature_extraction/FeatureMerger.h>

#include "../EnclosingEllipsoid.h"


// test includes
#define BOOST_TEST_MODULE Navigation::ArmarXObjects::LaserScannerFeatureExtraction
#define ARMARX_BOOST_TEST

#include <armarx/navigation/Test.h>

namespace armarx::navigation::components::laser_scanner_feature_extraction
{

    BOOST_AUTO_TEST_CASE(insideConvexPolyTest)
    {
        std::vector<Eigen::Vector2f> polygon;
        polygon.push_back({0, 0});
        polygon.push_back({1, 1});
        polygon.push_back({0, 2});
        polygon.push_back({-1, 1});

        BOOST_CHECK(FeatureMerger::insideConvexPoly({0, 1}, polygon));
        BOOST_CHECK(FeatureMerger::insideConvexPoly({0.5, 1}, polygon));
        BOOST_CHECK(FeatureMerger::insideConvexPoly({-0.5, 1}, polygon));

        BOOST_CHECK(FeatureMerger::insideConvexPoly({0, 2}, polygon));
        BOOST_CHECK(FeatureMerger::insideConvexPoly({1, 1}, polygon));

        BOOST_CHECK(FeatureMerger::insideConvexPoly({-0.5, 0.5}, polygon));
        BOOST_CHECK(FeatureMerger::insideConvexPoly({-0.5, 0.51}, polygon));
        BOOST_CHECK(!FeatureMerger::insideConvexPoly({-0.5, 0.49}, polygon));

        BOOST_CHECK(!FeatureMerger::insideConvexPoly({1.1, 1}, polygon));
        BOOST_CHECK(!FeatureMerger::insideConvexPoly({0, 2.1}, polygon));
        BOOST_CHECK(!FeatureMerger::insideConvexPoly({0, -0.1}, polygon));
        BOOST_CHECK(!FeatureMerger::insideConvexPoly({-1, 2}, polygon));
    }

} // namespace armarx::navigation::components::laser_scanner_feature_extraction
