/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    Navigation::ArmarXObjects::NavigationMemory
 * @author     Rainer Kartmann ( rainer dot kartmann at kit dot edu )
 * @date       2021
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

#include <map>
#include <memory>
#include <string>
#include <vector>

#include <SimoxUtility/color/Color.h>

#include <ArmarXCore/core/time/DateTime.h>
#include <ArmarXCore/core/time/Duration.h>

#include "RobotAPI/components/ArViz/Client/ScopedClient.h"
#include "RobotAPI/libraries/armem/server/wm/memory_definitions.h"
#include <RobotAPI/components/ArViz/Client/Client.h>
#include <RobotAPI/components/ArViz/Client/Layer.h>
#include <RobotAPI/libraries/ArmarXObjects/ObjectInfo.h>
#include <RobotAPI/libraries/ArmarXObjects/ObjectPoseClient.h>
#include <RobotAPI/libraries/ArmarXObjects/forward_declarations.h>
#include <RobotAPI/libraries/armem/core/forward_declarations.h>

#include <armarx/navigation/algorithms/Room.h>

namespace armarx::navigation::graph
{
    class GraphVisu;
}

namespace armarx::navigation::memory
{

    class Visu
    {
    public:
        Visu(viz::Client arviz,
             const armem::server::wm::CoreSegment& locSegment,
             const armem::server::wm::CoreSegment& graphSegment,
             const armem::server::wm::CoreSegment& costmapSegment,
             const armem::server::wm::CoreSegment& humanSegment,
             const armem::server::wm::CoreSegment& roomsSegment);
        ~Visu();


        void drawLocations(std::vector<viz::Layer>& layers, bool enabled);
        void drawLocations(objpose::ObjectPoseClient& objClient,
                           std::vector<viz::Layer>& layers,
                           bool enabled);

        void drawGraphs(std::vector<viz::Layer>& layers, bool enabled);
        void drawGraphs(objpose::ObjectPoseClient& objClient,
                        std::vector<viz::Layer>& layers,
                        bool enabled);
        void drawCostmaps(std::vector<viz::Layer>& layers, bool enabled, float zOffset);
        void drawHumans(std::vector<viz::Layer>& layers,
                        bool enabled,
                        bool visuTransparent,
                        Duration maxAge);

        void drawRooms(std::vector<viz::Layer>& layers, bool enabled);


        viz::ScopedClient arviz;

        const armem::server::wm::CoreSegment& locSegment;
        const armem::server::wm::CoreSegment& graphSegment;
        const armem::server::wm::CoreSegment& costmapSegment;
        const armem::server::wm::CoreSegment& humanSegment;
        const armem::server::wm::CoreSegment& roomsSegment;

        std::unique_ptr<navigation::graph::GraphVisu> visu;

    private:
        void
        drawRoom(viz::Layer& layer, const algorithms::Room& room, const simox::Color& color) const;

        void drawLocations(const objpose::ObjectPoseMap& objects,
                           const std::vector<ObjectInfo>& info,
                           viz::Layer& layer);
        void drawGraphs(const objpose::ObjectPoseMap& objects,
                        const std::vector<ObjectInfo>& info,
                        std::vector<viz::Layer>& layers,
                        bool enabled);


        std::map<std::string, DateTime> lastCostmapVisualization_;
    };


} // namespace armarx::navigation::memory
