/**
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Reister ( fabian dot reister at kit dot edu )
 * @author     Christian R. G. Dreher ( c dot dreher at kit dot edu )
 * @date       2021
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

#include <map>
#include <optional>
#include <string>
#include <vector>

#include <VirtualRobot/VirtualRobot.h>

#include <RobotAPI/libraries/ArmarXObjects/ObjectInfo.h>
#include <RobotAPI/libraries/ArmarXObjects/ObjectPose.h> // IWYU pragma: keep
#include <RobotAPI/libraries/ArmarXObjects/forward_declarations.h>

#include <armarx/navigation/algorithms/Costmap.h>
#include <armarx/navigation/core/location.h>

namespace armarx::navigation::core
{


    struct StaticScene
    {
        VirtualRobot::SceneObjectSetPtr objects;
        objpose::ObjectPoseMap objectMap;
        std::vector<ObjectInfo> objectInfo;

        std::optional<algorithms::Costmap> distanceToObstaclesCostmap;

        std::map<std::string, core::Location> locations;
    };


} // namespace armarx::navigation::core
