#include "LocalPlannerFactory.h"

#include <ArmarXCore/core/exceptions/local/ExpressionException.h>
#include <ArmarXCore/util/CPPUtility/trace.h>

#include <RobotAPI/libraries/aron/core/data/variant/container/Dict.h>
#include <RobotAPI/libraries/aron/core/data/variant/primitive/String.h>

#include <armarx/navigation/core/constants.h>
#include <armarx/navigation/core/fwd.h>
#include <armarx/navigation/factories/fwd.h>
#include <armarx/navigation/local_planning/core.h>

#ifdef TIMED_ELASTIC_BANDS_ENABLED
#include <armarx/navigation/local_planning/TimedElasticBands.h>
#endif

namespace armarx::navigation::fac
{
    local_planning::LocalPlannerPtr
    LocalPlannerFactory::create(const aron::data::DictPtr& params, const core::Scene& ctx)
    {
        ARMARX_TRACE;
        namespace layer = local_planning;

        if (not params)
        {
            return nullptr;
        }

        // algo name
        const auto algoName = aron::data::String::DynamicCast(params->getElement(core::NAME_KEY));
        ARMARX_CHECK_NOT_NULL(algoName);
        const layer::Algorithms algo = layer::AlgorithmNames.from_name(algoName->getValue());

        // algo params
        const auto algoParams = aron::data::Dict::DynamicCast(params->getElement(core::PARAMS_KEY));
        ARMARX_CHECK_NOT_NULL(algoParams);

        local_planning::LocalPlannerPtr localPlanner;
        switch (algo)
        {
            case local_planning::Algorithms::TimedElasticBands:
#ifdef TIMED_ELASTIC_BANDS_ENABLED
                localPlanner = std::make_shared<local_planning::TimedElasticBands>(
                    local_planning::TimedElasticBandsParams::FromAron(algoParams), ctx);
#endif

                break;
        }

        return localPlanner;
    }
} // namespace armarx::navigation::fac
