#include "SafetyGuardFactory.h"

#include <memory>

#include <ArmarXCore/core/exceptions/local/ExpressionException.h>
#include <ArmarXCore/core/logging/Logging.h>

#include <RobotAPI/libraries/aron/core/data/variant/container/Dict.h>
#include <RobotAPI/libraries/aron/core/data/variant/primitive/String.h>

#include <armarx/navigation/core/constants.h>
#include <armarx/navigation/core/types.h>
#include <armarx/navigation/safety_guard/LaserBasedProximity.h>
#include <armarx/navigation/safety_guard/SafetyGuard.h>
#include <armarx/navigation/safety_guard/core.h>

namespace armarx::navigation::fac
{
    safety_guard::SafetyGuardPtr
    SafetyGuardFactory::create(const aron::data::DictPtr& params,
                               const core::Scene& scene,
                               const safety_guard::SafetyGuard::Context& ctx)
    {
        namespace layer = safety_guard;

        if (not params)
        {
            return nullptr;
        }

        // algo name
        const auto algoName = aron::data::String::DynamicCast(params->getElement(core::NAME_KEY));
        ARMARX_CHECK_NOT_NULL(algoName);
        const layer::Algorithms algo = layer::AlgorithmNames.from_name(algoName->getValue());

        // algo params
        const auto algoParams = aron::data::Dict::DynamicCast(params->getElement(core::PARAMS_KEY));
        ARMARX_CHECK_NOT_NULL(algoParams);

        safety_guard::SafetyGuardPtr controller;
        switch (algo)
        {
            case safety_guard::Algorithms::LaserBasedProximity:
                ARMARX_INFO << "safety_guard::Algorithms::LaserBasedProximity";
                controller = std::make_shared<safety_guard::LaserBasedProximity>(
                    safety_guard::LaserBasedProximityParams::FromAron(algoParams), scene, ctx);
                break;
            default:
                ARMARX_INFO << "safety_guard::Algorithms::none";
                break;
        }

        return controller;
    }
} // namespace armarx::navigation::fac
