/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Rainer Kartmann ( rainer dot kartmann at kit dot edu )
 * @date       2021
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

#include <memory>
#include <optional>
#include <string>
#include <vector>

#include <QWidget>
#include <qobjectdefs.h>
#include <qwidget.h>

#include <Eigen/Core>

#include <RobotAPI/libraries/ArmarXObjects/ObjectInfo.h>
#include <RobotAPI/libraries/ArmarXObjects/forward_declarations.h>


class QLabel;
class QPushButton;

namespace IceInternal
{
    template <typename T>
    class ProxyHandle;
}

namespace IceProxy::armarx::objpose
{
    class ObjectPoseStorageInterface;
}

namespace armarx
{
    class ManagedIceObject;
    using ObjectPoseStorageInterfacePrx =
        ::IceInternal::ProxyHandle<::IceProxy::armarx::objpose::ObjectPoseStorageInterface>;
} // namespace armarx

namespace armarx::navigation::qt_plugins::location_graph_editor::objectposes
{

    class Connection
    {
    public:
        Connection(ObjectPoseStorageInterfacePrx objectPoseStorageComponent);
        ~Connection();

        ObjectPoseStorageInterfacePrx getObjectPoseStorageComponent() const;
        std::string getConnectedName() const;

        objpose::ObjectPoseClient& getObjectClient();

        objpose::ObjectPoseMap getObjectPoseMap();
        std::vector<ObjectInfo> getObjectInfo();

    private:
        std::unique_ptr<ObjectPoseStorageInterfacePrx> _objectPoseStorageComponent;
        std::unique_ptr<objpose::ObjectPoseClient> _objectPoseClient;
    };

} // namespace armarx::navigation::qt_plugins::location_graph_editor::objectposes

namespace armarx::navigation::qt_plugins::location_graph_editor

{

    class ObjectPoseClientWidget : public QWidget
    {
        Q_OBJECT
        using This = ObjectPoseClientWidget;

    public:
        ObjectPoseClientWidget(ManagedIceObject& component, QWidget* parent = nullptr);


        bool isConnected() const;
        objectposes::Connection& connection();


    signals:

        void connected();


    public slots:

        void connectToObjectPoseComponent();


    private:
        ManagedIceObject& _component;
        std::optional<objectposes::Connection> _connection;

        QLabel* _statusLabel = nullptr;
        QPushButton* _connectButton = nullptr;
    };

} // namespace armarx::navigation::qt_plugins::location_graph_editor
