#include "aron_conversions.h"

#include <armarx/navigation/conversions/eigen.h>
#include <armarx/navigation/human/aron/Human.aron.generated.h>
#include <armarx/navigation/human/types.h>

#include <range/v3/range/conversion.hpp>
#include <range/v3/view/transform.hpp>

namespace armarx::navigation::human
{
    void
    toAron(arondto::Human& dto, const Human& bo)
    {
        dto.pose = conv::to3D(bo.pose).matrix();
        dto.linearVelocity = conv::to3D(bo.linearVelocity);
        dto.detectionTime = bo.detectionTime;
    }

    void
    fromAron(const arondto::Human& dto, Human& bo)
    {
        bo.pose = conv::to2D(core::Pose(dto.pose));
        bo.linearVelocity = conv::to2D(dto.linearVelocity);
        bo.detectionTime = dto.detectionTime;
    }

    void
    toAron(arondto::HumanGroup& dto, const HumanGroup& bo)
    {
        dto.shape = bo.shape.vertices |
                    ranges::views::transform([](const Eigen::Vector2f& boVer) -> Eigen::Vector3f
                                             { return conv::to3D(boVer); }) |
                    ranges::to_vector;

        dto.humans = bo.humans |
                     ranges::views::transform(
                         [](const Human& boHuman) -> arondto::Human
                         {
                             arondto::Human dtoHuman;
                             toAron(dtoHuman, boHuman);
                             return dtoHuman;
                         }) |
                     ranges::to_vector;
        dto.detectionTime = bo.detectionTime;
    }

    void
    fromAron(const arondto::HumanGroup& dto, HumanGroup& bo)
    {
        bo.shape.vertices =
            dto.shape |
            ranges::views::transform([](const Eigen::Vector3f& dtoVer) -> Eigen::Vector2f
                                     { return conv::to2D(dtoVer); }) |
            ranges::to_vector;

        bo.humans = dto.humans |
                    ranges::views::transform(
                        [](const arondto::Human& dtoHuman) -> Human
                        {
                            Human boHuman;
                            fromAron(dtoHuman, boHuman);
                            return boHuman;
                        }) |
                    ranges::to_vector;
        bo.detectionTime = dto.detectionTime;
    }

} // namespace armarx::navigation::human
