#include "types.h"

#include <ArmarXCore/core/exceptions/local/ExpressionException.h>
#include <ArmarXCore/core/time/DateTime.h>

#include <RobotAPI/libraries/aron/core/data/variant/container/Dict.h>

#include <armarx/navigation/core/basic_types.h>
#include <armarx/navigation/human/aron/Human.aron.generated.h>
#include <armarx/navigation/human/aron_conversions.h>

namespace armarx::navigation::human
{
    core::Pose2D
    Human::estimateAt(const DateTime& time) const
    {
        double dt = (time - detectionTime).toSecondsDouble();
        core::Pose2D estimation{pose};
        estimation.translation() += linearVelocity * dt;
        return estimation;
    }

    aron::data::DictPtr
    Human::toAron() const
    {
        arondto::Human dto;
        human::toAron(dto, *this);
        return dto.toAron();
    }

    Human
    Human::FromAron(const aron::data::DictPtr& dict)
    {
        ARMARX_CHECK_NOT_NULL(dict);
        arondto::Human dto;
        dto.fromAron(dict);

        Human bo;
        fromAron(dto, bo);
        return bo;
    }

    aron::data::DictPtr
    HumanGroup::toAron() const
    {
        arondto::HumanGroup dto;
        human::toAron(dto, *this);
        return dto.toAron();
    }

    HumanGroup
    HumanGroup::FromAron(const aron::data::DictPtr& dict)
    {
        ARMARX_CHECK_NOT_NULL(dict);
        arondto::HumanGroup dto;
        dto.fromAron(dict);

        HumanGroup bo;
        fromAron(dto, bo);
        return bo;
    }

} // namespace armarx::navigation::human
