/**
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Reister ( fabian dot reister at kit dot edu )
 * @author     Christian R. G. Dreher ( c dot dreher at kit dot edu )
 * @date       2021
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

#include <optional>
#include <string>

#include <armarx/navigation/core/Trajectory.h>
#include <armarx/navigation/core/types.h>
#include <armarx/navigation/local_planning/LocalPlanner.h>
#include <armarx/navigation/local_planning/TebObstacleManager.h>
#include <armarx/navigation/local_planning/TimedElasticBandsParams.h>
#include <armarx/navigation/local_planning/aron/TimedElasticBands.aron.generated.h>

#include <teb_local_planner/homotopy_class_planner.h>
#include <teb_local_planner/teb_config.h>

namespace armarx::navigation::local_planning
{
    class TimedElasticBands : virtual public LocalPlanner
    {
    public:
        using Params = TimedElasticBandsParams;

        TimedElasticBands(const Params& params, const core::Scene& scene);
        ~TimedElasticBands() override = default;

        void init(const core::GlobalTrajectory& initialTrajectory);

        std::optional<LocalPlannerResult> plan(const core::GlobalTrajectory& goal) override;

    private:
        void readConfig(arondto::TimedElasticBandsParams& target, const std::string& robotName);
        void fillObstacles();
        void setTebCostmap();

    protected:
        Params params;

    private:
        const core::Scene& scene;

        teb_local_planner::TebConfig cfg_;
        teb_local_planner::ObstContainer teb_obstacles;
        TebObstacleManager obstManager;
        teb_local_planner::PoseSequence teb_globalPath;
        std::optional<teb_local_planner::Costmap> teb_costmap;
        std::unique_ptr<teb_local_planner::HomotopyClassPlanner> hcp_{nullptr};
    };
} // namespace armarx::navigation::local_planning
