/**
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Tobias Gröger ( tobias dot groeger at student dot kit dot edu )
 * @date       2022
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

#include <armarx/navigation/algorithms/Costmap.h>
#include <armarx/navigation/core/Trajectory.h>
#include <armarx/navigation/core/basic_types.h>
#include <armarx/navigation/human/shapes.h>
#include <armarx/navigation/human/types.h>

#include <geometry_msgs/Twist.h>
#include <teb_local_planner/costmap.h>
#include <teb_local_planner/penalty_models.h>
#include <teb_local_planner/pose_se2.h>
#include <teb_local_planner/timed_elastic_band.h>

namespace armarx::navigation::conv
{

    Eigen::Vector2d toRos2D(const Eigen::Vector2f& vec);

    Eigen::Vector2d toRos(const Eigen::Vector3f& vec);
    Eigen::Vector3f fromRos(const Eigen::Vector2d& vec);

    teb_local_planner::PoseSE2 toRos(const core::Pose& pose);
    teb_local_planner::PoseSE2 toRos(const core::Pose2D& pose);

    core::Pose fromRos(const teb_local_planner::PoseSE2& pose);

    geometry_msgs::Twist toRos(const core::Twist& velocity);

    teb_local_planner::TimedElasticBand toRos(const core::GlobalTrajectory& trajectory);

    core::LocalTrajectory fromRos(const teb_local_planner::TimedElasticBand& teb);

    human::shapes::Ellipse toRos(const human::shapes::Ellipse& ellipse);

    teb_local_planner::Costmap toRos(const algorithms::Costmap& costmap);

    teb_local_planner::PenaltyModelPtr toRos(const human::LinearPenaltyModel& model);

    teb_local_planner::PenaltyModelPtr toRos(const human::ExponentialPenaltyModel& model);

} // namespace armarx::navigation::conv
