#include "aron_conversions.h"

#include <RobotAPI/libraries/aron/common/aron_conversions/eigen.h>
#include <RobotAPI/libraries/aron/core/aron_conversions.h>

#include <armarx/navigation/memory/aron/LaserScannerFeatures.aron.generated.h>

#include "types.h"

namespace armarx::navigation::memory
{


    /************ toAron ************/

    // auto toAron(const LaserScan& laserScan, aron::LaserScan& aronLaserScan)
    // {
    //     aronLaserScan.scan = toAron(laserScan);
    // }


    // LaserScannerFeatures

    void
    toAron(arondto::Circle& dto, const Circle& bo)
    {
        dto.center = bo.center;
        dto.radius = bo.radius;
    }

    void
    toAron(arondto::Ellipsoid& dto, const Ellipsoid& bo)
    {
        dto.globalPose = bo.pose.matrix();
        dto.radii = bo.radii;
    }

    void
    toAron(arondto::LaserScannerFeature& dto, const LaserScannerFeature& bo)
    {
        toAron(dto.circle, bo.circle);
        dto.convexHull = bo.convexHull;
        toAron(dto.ellipsoid, bo.ellipsoid);
        dto.points = bo.points;
    }

    void
    toAron(arondto::LaserScannerFeatures& dto, const LaserScannerFeatures& bo)
    {
        aron::toAron(dto.frame, bo.frame);
        aron::toAron(dto.frameGlobalPose, bo.frameGlobalPose);
        aron::toAron(dto.features, bo.features);
    }

    void
    fromAron(const arondto::Circle& dto, Circle& bo)
    {
        bo.center = dto.center;
        bo.radius = dto.radius;
    }

    void
    fromAron(const arondto::Ellipsoid& dto, Ellipsoid& bo)
    {
        bo.pose = dto.globalPose;
        bo.radii = dto.radii;
    }

    void
    fromAron(const arondto::LaserScannerFeature& dto, LaserScannerFeature& bo)
    {
        bo.convexHull = dto.convexHull;
        fromAron(dto.circle, bo.circle);
        fromAron(dto.ellipsoid, bo.ellipsoid);

        // bo.chain = dto.chain;
        bo.points = dto.points;
    }

    void
    fromAron(const arondto::LaserScannerFeatures& dto, LaserScannerFeatures& bo)
    {
        aron::fromAron(dto.frame, bo.frame);
        aron::fromAron(dto.frameGlobalPose, bo.frameGlobalPose);
        aron::fromAron(dto.features, bo.features);
    }

} // namespace armarx::navigation::memory
