/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Reister ( fabian dot reister at kit dot edu )
 * @date       2021
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

#include <string>
#include <vector>

#include <ArmarXCore/core/time/DateTime.h>

#include <RobotAPI/libraries/armem/client/query/Builder.h>
#include <RobotAPI/libraries/armem/client/util/SimpleReaderBase.h>

#include <armarx/navigation/memory/types.h>

namespace armarx::navigation::memory::client::laser_scanner_features
{


    /**
    * @defgroup Component-ExampleClient ExampleClient
    * @ingroup RobotAPI-Components
    * A description of the component ExampleClient.
    *
    * @class ExampleClient
    * @ingroup Component-ExampleClient
    * @brief Brief description of class ExampleClient.
    *
    * Detailed description of class ExampleClient.
    */
    class Reader : virtual public armarx::armem::client::util::SimpleReaderBase
    {
    public:
        using armarx::armem::client::util::SimpleReaderBase::SimpleReaderBase;

        virtual ~Reader();

        struct Query
        {
            std::string providerName;

            std::string name; // sensor name

            armarx::core::time::DateTime timestamp;

            // std::vector<std::string> sensorList;
        };

        struct Result
        {
            std::vector<LaserScannerFeatures> features;

            // std::vector<std::string> sensors;

            enum Status
            {
                Error,
                Success
            } status;

            std::string errorMessage;
        };

        Result queryData(const Query& query) const;

    protected:
        armarx::armem::client::query::Builder buildQuery(const Query& query) const;

        std::string propertyPrefix() const override;
        Properties defaultProperties() const override;
    };

} // namespace armarx::navigation::memory::client::laser_scanner_features
