/**
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Reister ( fabian dot reister at kit dot edu )
 * @date       2021
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

#include <string>

#include <ArmarXCore/core/time/DateTime.h>

#include <RobotAPI/libraries/armem/client/Query.h>
#include <RobotAPI/libraries/armem/client/util/SimpleReaderBase.h>
#include <RobotAPI/libraries/armem/core/wm/memory_definitions.h>

#include <armarx/navigation/core/Trajectory.h>

namespace armarx::navigation::memory::client::stack_result
{

    class Reader : virtual public armem::client::util::SimpleReaderBase
    {
    public:
        using armem::client::util::SimpleReaderBase::SimpleReaderBase;

        struct Query
        {
            DateTime timestamp;
            std::string clientID;
        };

        struct LocalPlannerResult
        {
            armarx::navigation::core::LocalTrajectory trajectory;

            enum Status
            {
                Success,
                NoData,
                Error
            } status;

            std::string errorMessage = "";

            operator bool() const noexcept
            {
                return status == Status::Success;
            }
        };

        struct GlobalPlannerResult
        {
            armarx::navigation::core::GlobalTrajectory trajectory;

            enum Status
            {
                Success,
                NoData,
                Error
            } status;

            std::string errorMessage = "";

            operator bool() const noexcept
            {
                return status == Status::Success;
            }
        };

        GlobalPlannerResult queryGlobalPlannerResult(const Query& query);
        LocalPlannerResult queryLocalPlannerResult(const Query& query);

        std::string propertyPrefix() const override;
        Properties defaultProperties() const override;

    protected:
    private:
        armarx::navigation::core::LocalTrajectory
        asLocalPlannerResult(const armem::wm::ProviderSegment& providerSegment);

        armarx::navigation::core::GlobalTrajectory
        asGlobalPlannerResult(const armem::wm::ProviderSegment& providerSegment);

        armarx::armem::client::query::Builder
        buildLocalPlannerResultQuery(const Query& query) const;

        armarx::armem::client::query::Builder
        buildGlobalPlannerResultQuery(const Query& query) const;
    };
} // namespace armarx::navigation::memory::client::stack_result
