/**
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Reister ( fabian dot reister at kit dot edu )
 * @date       2023
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

#include <armarx/navigation/algorithms/Costmap.h>
#include <armarx/navigation/algorithms/Room.h>
#include <armarx/navigation/core/basic_types.h>

namespace armarx::viz
{
    class Client;
}

namespace armarx::navigation::rooms
{

    class RoomNavigationTargetCreator
    {
    public:
        struct Params
        {
            float distanceToRoomEntry = 1000;
        };

        RoomNavigationTargetCreator(const Params& params);

        struct Result
        {
            core::Position global_P_roomEntry;
            core::Position global_P_robot; // where the robot should be placed
        };

        Result getClosestPositionOutsideOfRoom(const algorithms::Costmap& obstacleDistancesCostmap,
                                               const algorithms::Room& room,
                                               const core::Position& global_P_robot,
                                               viz::Client* arviz = nullptr) const;

    protected:
    private:
        const Params params;
    };
} // namespace armarx::navigation::rooms
