/**
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Reister ( fabian dot reister at kit dot edu )
 * @author     Christian R. G. Dreher ( c dot dreher at kit dot edu )
 * @date       2021
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

#include <experimental/memory>
#include <memory>

#include <VirtualRobot/VirtualRobot.h>

#include <RobotAPI/components/ArViz/Client/ScopedClient.h>
#include <RobotAPI/libraries/aron/core/data/variant/forward_declarations.h>

#include <armarx/navigation/core/DynamicScene.h>
#include <armarx/navigation/core/StaticScene.h>
#include <armarx/navigation/core/Trajectory.h>
#include <armarx/navigation/core/types.h>

#include "core.h"

namespace armarx::viz
{
    struct Client;
}

namespace armarx::navigation::safety_guard
{
    struct SafetyGuardResult
    {
        core::TwistLimits twistLimits;

        static SafetyGuardResult
        NoLimits()
        {
            return {.twistLimits = {.linear = std::numeric_limits<float>::max(),
                                    .angular = std::numeric_limits<float>::max()}};
        }

        static SafetyGuardResult
        ZeroLimits()
        {
            return {.twistLimits = {.linear = 0, .angular = 0}};
        }
    };

    struct SafetyGuardParams
    {
        virtual ~SafetyGuardParams() = default;

        virtual Algorithms algorithm() const = 0;
        virtual aron::data::DictPtr toAron() const = 0;
    };

    class SafetyGuard
    {
    public:
        struct Context
        {
            std::experimental::observer_ptr<viz::Client> arviz;
        };

        SafetyGuard(const core::Scene& scene, const Context& ctx);
        virtual ~SafetyGuard() = default;

        virtual SafetyGuardResult computeSafetyLimits() = 0;

    protected:
        const core::Scene& scene;
        viz::ScopedClient viz;

    private:
        const Context context;
    };

    using SafetyGuardPtr = std::shared_ptr<SafetyGuard>;
} // namespace armarx::navigation::safety_guard
