/**
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Reister ( fabian dot reister at kit dot edu )
 * @date       2022
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#include "aron_conversions.h"

#include <RobotAPI/libraries/aron/common/aron_conversions/eigen.h>
#include <RobotAPI/libraries/aron/core/aron_conversions.h>

#include <armarx/navigation/core/aron_conversions.h>

namespace armarx::navigation::safety_guard
{
    void
    fromAron(const arondto::ProximityFieldParams& dto, ProximityFieldParams& bo)
    {
        aron::fromAron(dto.reduceVelocity, bo.reduceVelocity);

        aron::fromAron(dto.minimumDistance, bo.minDistance);
        aron::fromAron(dto.maximumDistance, bo.maxDistance);
        aron::fromAron(dto.k, bo.k);
        aron::fromAron(dto.lambda, bo.lambda);
    }

    void
    toAron(arondto::ProximityFieldParams& dto, const ProximityFieldParams& bo)
    {
        aron::toAron(dto.reduceVelocity, bo.reduceVelocity);

        aron::toAron(dto.minimumDistance, bo.minDistance);
        aron::toAron(dto.maximumDistance, bo.maxDistance);
        aron::toAron(dto.k, bo.k);
        aron::toAron(dto.lambda, bo.lambda);
    }

    void
    fromAron(const arondto::LaserBasedProximityParams& dto, LaserBasedProximityParams& bo)
    {
        aron::fromAron(dto.enableHumans, bo.enableHumans);
        aron::fromAron(dto.enableLaserScanner, bo.enableLaserScanners);

        core::fromAron(dto.maxVelocity, bo.maxVelocity);

        fromAron(dto.humanProximityField, bo.humanProximityField);
        fromAron(dto.laserScannerProximityField, bo.laserScannerProximityField);
    }

    void
    toAron(arondto::LaserBasedProximityParams& dto, const LaserBasedProximityParams& bo)
    {
        aron::toAron(dto.enableHumans, bo.enableHumans);
        aron::toAron(dto.enableLaserScanner, bo.enableLaserScanners);

        core::toAron(dto.maxVelocity, bo.maxVelocity);

        toAron(dto.humanProximityField, bo.humanProximityField);
        toAron(dto.laserScannerProximityField, bo.laserScannerProximityField);
    }


} // namespace armarx::navigation::safety_guard
