/**
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Reister ( fabian dot reister at kit dot edu )
 * @date       2022
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

#include <RobotAPI/libraries/aron/core/codegeneration/cpp/AronGeneratedClass.h>

#include <armarx/navigation/safety_guard/LaserBasedProximity.h>
#include <armarx/navigation/safety_guard/aron/LaserBasedProximityParams.aron.generated.h>

namespace armarx
{
    template <typename AronType, typename T>
    T
    fromAron(const ::armarx::aron::data::dto::DictPtr& dto)
    {
        static_assert(
            std::is_base_of_v<armarx::aron::codegenerator::cpp::AronGeneratedClass, AronType>,
            "AronType must be an ARON generated type");

        const auto dtoConfig = AronType::FromAron(dto);

        T config;
        fromAron(dtoConfig, config);

        return config;
    }
} // namespace armarx

namespace armarx::navigation::safety_guard
{

    void fromAron(const arondto::ProximityFieldParams& dto, ProximityFieldParams& bo);
    void toAron(arondto::ProximityFieldParams& dto, const ProximityFieldParams& bo);

    void fromAron(const arondto::LaserBasedProximityParams& dto, LaserBasedProximityParams& bo);
    void toAron(arondto::LaserBasedProximityParams& dto, const LaserBasedProximityParams& bo);

} // namespace armarx::navigation::safety_guard
