#pragma once

#include <armarx/navigation/core/Trajectory.h>
#include <armarx/navigation/core/types.h>

namespace armarx::navigation::server
{

    /**
     * @brief An executer the server navigator will use to send its control commands to.
     */
    class ExecutorInterface
    {

    public:
        virtual ~ExecutorInterface() = default;

        virtual void execute(const core::LocalTrajectory& trajectory,
                             bool activateController = false) = 0;
        virtual void execute(const core::GlobalTrajectory& trajectory,
                             bool activateController = false) = 0;


        enum class ControllerType
        {
            GlobalTrajectory,
            LocalTrajectory,
            None
        };

        virtual void start(ControllerType controllerType) = 0;
        virtual void stop() = 0;

        virtual void ensureIsActive(ControllerType controllerType) = 0;

        virtual void updateVelocityLimits(const core::TwistLimits& limits) = 0;
    };

} // namespace armarx::navigation::server
