#pragma once

#include <memory>
#include <string>

#include <armarx/control/client/ComponentPlugin.h>
#include <armarx/control/client/ControllerWrapper.h>
#include <armarx/navigation/common/controller_types.h>
#include <armarx/navigation/core/Trajectory.h>
#include <armarx/navigation/core/types.h>
#include <armarx/navigation/platform_controller/controller_descriptions.h>
#include <armarx/navigation/server/execution/ExecutorInterface.h>

namespace armarx::control::client
{
    class ComponentPlugin;
}

namespace armarx::navigation::server
{

    /**
     * @brief The PlatformUnitExecutor class
     *
     * TODO: Should be renamed to whatever the new unit will be called which takes Mat4f
     * as input instead of X/Y/Yaw, so that we have a generic interface.
     */
    class PlatformControllerExecutor : virtual public ExecutorInterface
    {

    public:
        using ControllerComponentPlugin = armarx::control::client::ComponentPlugin;

        struct Properties
        {
            std::string robotName;
            bool rtUnitDynamicallyLoadLibraries = false;
        };

        PlatformControllerExecutor(ControllerComponentPlugin& controllerComponentPlugin,
                                   const Properties& properties);
        ~PlatformControllerExecutor() override;

        void execute(const core::LocalTrajectory& trajectory,
                     bool activateController = false) override;
        void execute(const core::GlobalTrajectory& trajectory,
                     bool activateController = false) override;

        void start(ControllerType controllerType) override;

        void ensureIsActive(ControllerType controllerType) override;

        void stop() override;

        void updateVelocityLimits(const core::TwistLimits& limits) override;


    private:
        ControllerComponentPlugin& controllerPlugin_;

        const Properties properties_;


        std::unique_ptr<armarx::control::client::ControllerWrapper<
            armarx::navigation::common::ControllerType::PlatformGlobalTrajectory>>
            globalTrajCtrl_;

        std::unique_ptr<armarx::control::client::ControllerWrapper<
            armarx::navigation::common::ControllerType::PlatformLocalTrajectory>>
            localTrajCtrl_;

        control::client::ControllerDescription<
            armarx::navigation::common::ControllerType::PlatformGlobalTrajectory>::AronDTO
            globalTrajCtrlInitialConfig_;

        control::client::ControllerDescription<
            armarx::navigation::common::ControllerType::PlatformLocalTrajectory>::AronDTO
            localTrajCtrlInitialConfig_;


        ControllerType lastActiveController_ = ControllerType::None;
    };

} // namespace armarx::navigation::server
