/**
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Reister ( fabian dot reister at kit dot edu )
 * @date       2022
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

#include <armarx/navigation/algorithms/Costmap.h>
#include <armarx/navigation/core/Trajectory.h>
#include <armarx/navigation/core/basic_types.h>
#include <armarx/navigation/human/types.h>

namespace armarx::navigation::human::simulation
{

    struct SimulatedHumanParams
    {
        float goalDistanceThreshold = 100; // [mm]

        float minLinearVelocity = 100; // [mm/s]
        float maxLinearVelocity = 200; // [mm/s]
    };

    class SimulatedHuman
    {
    public:
        using Params = SimulatedHumanParams;

        SimulatedHuman(const algorithms::Costmap& distanceField, const Params& params = Params());

        Human update();

        human::Human&
        human()
        {
            return human_;
        }

        core::GlobalTrajectory&
        trajectory()
        {
            return globalTrajectory_;
        }

        const core::GlobalTrajectory&
        trajectory() const
        {
            return globalTrajectory_;
        }

        bool goalReached() const;

        /**
         * @brief resets the human to the given pose and replans the global optimal trajectory
         * 
         * @param pose 
         */
        void reinitialize(const core::Pose2D& pose);

    protected:
        void initialize();

        void step();

        enum class State
        {
            Idle,
            Walking,
            GoalReached
        };


    private:
        const algorithms::Costmap distanceField_;

        State state_ = State::Idle;

        human::Human human_;

        core::GlobalTrajectory globalTrajectory_;

        const Params params_;
    };

} // namespace armarx::navigation::human::simulation
