/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Reister ( fabian dot reister at kit dot edu )
 * @date       2023
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

#include <optional>
#include <string>

#include <ArmarXCore/core/time/Duration.h>

#include "RobotAPI/libraries/armem_robot_state/client/common/VirtualRobotReader.h"
#include <RobotAPI/components/ArViz/Client/Client.h>
#include <RobotAPI/libraries/skills/core/Skill.h>
#include <RobotAPI/libraries/skills/core/SkillDescription.h>
#include <RobotAPI/libraries/skills/provider/SimpleSpecializedSkill.h>

#include <armarx/navigation/client.h>
#include <armarx/navigation/client/services/IceNavigator.h>
#include <armarx/navigation/client/services/MemorySubscriber.h>
#include <armarx/navigation/core/basic_types.h>
#include <armarx/navigation/memory/client/costmap/Reader.h>
#include <armarx/navigation/memory/client/rooms/Reader.h>
#include <armarx/navigation/skills/aron/GuideHumanToRoom.aron.generated.h>
#include <armarx/navigation/skills/constants.h>

namespace armarx::navigation::skills
{
    class GuideHumanToRoom :
        virtual public armarx::skills::SimpleSpecializedSkill<arondto::GuideHumanToRoomParams>
    {
    public:
        using Params = arondto::GuideHumanToRoomParams;
        using Base = ::armarx::skills::SimpleSpecializedSkill<Params>;

        struct Services
        {
            client::IceNavigator& iceNavigator;
            armem::client::MemoryNameSystem& memoryNameSystem;

            armem::robot_state::VirtualRobotReader& virtualRobotReader;

            // objpose::ObjectPoseClient& objectPoseClient;

            memory::client::costmap::Reader& costmapReader;
            memory::client::rooms::Reader& roomsReader;

            viz::Client& arviz;
        };

        struct Properties
        {
            std::string distanceToObstacleCostmapProvider =
                "distance_to_obstacle_costmap_provider"; // FIXME
            std::string roomsProvider = "R003"; // FIXME
            std::string robotName = "Armar6"; // FIXME
        };

        GuideHumanToRoom(const Services& srv);


    private:
        ::armarx::skills::Skill::InitResult init(const Base::SpecializedInitInput& in) override;

        ::armarx::skills::Skill::MainResult main(const Base::SpecializedMainInput& in) override;

        void onStopRequested() override;

        core::Pose getTargetInFrontOfRoom(const std::string& roomName) const;

    public:
        static armarx::skills::SkillDescription
        DefaultSkillDescription()
        {
            ParamType defaultParameters;
            // TODO(): set parameter defaults

            return armarx::skills::SkillDescription{
                .skillId = {.skillName = constants::skill_names::GuideHumanToRoom},
                .description = "",
                .rootProfileDefaults = defaultParameters.toAron(),
                .timeout = armarx::core::time::Duration(),
                .parametersType = Params::ToAronType()};
        }

    protected:
    private:
        std::optional<Services> srv_;
        const Properties properties;

        // will be initialized in "init()"
        std::optional<client::MemorySubscriber> memorySubscriber;
        std::optional<client::Navigator> navigator;
    };

} // namespace armarx::navigation::skills
