#include "MoveXMeters.h"

#include <armarx/navigation/skills/constants.h>

namespace armarx::navigation::skills
{

    MoveXMeters::MoveXMeters(const Services& srv) : Base(srv, DefaultSkillDescription())
    {
    }

    Eigen::Isometry3f
    MoveXMeters::relativeTarget(const Base::SpecializedMainInput& in)
    {
        Eigen::Vector3f scaledVector = in.parameters.distanceMillimeters * Eigen::Vector3f::UnitY();
        return Eigen::Isometry3f(Eigen::Translation3f(in.parameters.direction * scaledVector));
    }

    armarx::skills::SkillDescription
    MoveXMeters::DefaultSkillDescription()
    {
        ParamType defaultParams;
        defaultParams.distanceMillimeters = 0;
        defaultParams.direction = Eigen::Quaternion<float, 0>::Identity();

        return armarx::skills::SkillDescription{
            .skillId = {.skillName = constants::skill_names::MoveXMeters},
            .description = "Relative movement in specified direction",
            .rootProfileDefaults = defaultParams.toAron(),
            .timeout = armarx::core::time::Duration::Minutes(1),
            .parametersType = ParamType::ToAronType()};
    }
} // namespace armarx::navigation::skills
