/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Reister ( fabian dot reister at kit dot edu )
 * @date       2023
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

#include <optional>

#include <ArmarXCore/core/time/Duration.h>

#include <RobotAPI/libraries/skills/core/Skill.h>
#include <RobotAPI/libraries/skills/core/SkillDescription.h>
#include <RobotAPI/libraries/skills/provider/SimpleSpecializedSkill.h>

#include <armarx/navigation/client.h>
#include <armarx/navigation/client/services/IceNavigator.h>
#include <armarx/navigation/client/services/MemorySubscriber.h>
#include <armarx/navigation/skills/aron/NavigateTo.aron.generated.h>
#include <armarx/navigation/skills/constants.h>

namespace armarx::navigation::skills
{
    class NavigateTo :
        virtual public armarx::skills::SimpleSpecializedSkill<arondto::NavigateToParams>
    {
    public:
        using Params = arondto::NavigateToParams;
        using Base = ::armarx::skills::SimpleSpecializedSkill<Params>;

        struct Services
        {
            client::IceNavigator& iceNavigator;
            armem::client::MemoryNameSystem& memoryNameSystem;
        };

        NavigateTo(const Services& srv);

    private:
        ::armarx::skills::Skill::InitResult init(const Base::SpecializedInitInput& in) override;

        ::armarx::skills::Skill::MainResult main(const Base::SpecializedMainInput& in) override;

        void onStopRequested() override;


    public:
        static armarx::skills::SkillDescription
        DefaultSkillDescription()
        {
            ParamType defaultParameters;
            // TODO(): set parameter defaults

            return armarx::skills::SkillDescription{
                .skillId = {.skillName = constants::skill_names::NavigateTo},
                .description = "",
                .rootProfileDefaults = defaultParameters.toAron(),
                .timeout = armarx::core::time::Duration::Hours(1),
                .parametersType = Params::ToAronType()};
        }

    protected:
    private:
        std::optional<Services> srv_;

        // will be initialized in "init()"
        std::optional<client::MemorySubscriber> memorySubscriber;
        std::optional<client::Navigator> navigator;
    };

} // namespace armarx::navigation::skills
