#pragma once

#include <RobotAPI/libraries/skills/core/Skill.h>
#include <RobotAPI/libraries/skills/core/SkillDescription.h>
#include <RobotAPI/libraries/skills/core/SkillID.h>
#include <RobotAPI/libraries/skills/provider/SimpleSpecializedSkill.h>

#include <armarx/navigation/skills/aron/NavigateToChargingStation.aron.generated.h>

namespace armarx::navigation::skills
{
    /**
     * @class NavigateToChargingStation
     * @ingroup Library-armarx_navigation_skills
     *
     * @brief Navigate to the charging station location and initiate charging.
     *
     * Subskills:
     * - armarx::navigation::skills::NavigateToNamedLocation
     *
     * @todo Rename, this is more general than just navigation.
     */
    class NavigateToChargingStation :
        virtual public armarx::skills::SimpleSpecializedSkill<
            arondto::NavigateToChargingStationParams>
    {
    public:
        using Params = arondto::NavigateToChargingStationParams;
        using Base = ::armarx::skills::SimpleSpecializedSkill<Params>;

        static ::armarx::skills::SkillDescription GetSkillDescription();

        struct Properties
        {
            struct SubSkillIds
            {
                SubSkillIds();

                armarx::skills::SkillID navigateToNamedLocation;
            };

            SubSkillIds subSkillIDs;
        };

        NavigateToChargingStation(const Properties& properties);

    private:
        // SpecializedSkill interface
        // Enable each function you want to override.
        // bool isAvailable(const SpecializedInitInput&) const override;
        // ::armarx::skills::Skill::InitResult init(const SpecializedInitInput&) override;
        ::armarx::skills::Skill::MainResult main(const SpecializedMainInput& in) override;
        // ::armarx::skills::Skill::ExitResult exit(const SpecializedExitInput&) override;


    protected:
    private:
        Properties properties;
    };

} // namespace armarx::navigation::skills
