/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Reister ( fabian dot reister at kit dot edu )
 * @date       2023
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

#include <experimental/memory>
#include <mutex>
#include <optional>

#include <RobotAPI/libraries/skills/core/Skill.h>
#include <RobotAPI/libraries/skills/core/SkillDescription.h>
#include <RobotAPI/libraries/skills/provider/SimpleSpecializedSkill.h>

#include <armarx/navigation/client.h>
#include <armarx/navigation/client/services/IceNavigator.h>
#include <armarx/navigation/client/services/MemorySubscriber.h>
#include <armarx/navigation/safety_guard/LaserBasedProximity.h>
#include <armarx/navigation/skills/aron/NavigateToLocation.aron.generated.h>

namespace armarx::navigation::skills
{
    class NavigateToLocation :
        virtual public armarx::skills::SimpleSpecializedSkill<arondto::NavigateToLocationParams>
    {
    public:
        using Params = arondto::NavigateToLocationParams;
        using Base = ::armarx::skills::SimpleSpecializedSkill<Params>;

        struct Services
        {
            client::IceNavigator& iceNavigator;

            armem::client::MemoryNameSystem& memoryNameSystem;
        };

        struct Properties
        {
            std::experimental::observer_ptr<std::mutex> safetyGuardParamsMutex;
            std::experimental::observer_ptr<safety_guard::LaserBasedProximityParams>
                safetyGuardParams;
        };

        NavigateToLocation(const Properties& properties, const Services& srv);

    private:
        ::armarx::skills::Skill::InitResult init(const Base::SpecializedInitInput& in) override;

        ::armarx::skills::Skill::MainResult main(const Base::SpecializedMainInput& in) override;

        void onStopRequested() override;


    public:
        static armarx::skills::SkillDescription DefaultSkillDescription();

    protected:
    private:
        Properties properties_;
        std::optional<Services> srv_;

        // will be initialized in "init()"
        std::optional<client::MemorySubscriber> memorySubscriber;
        std::optional<client::Navigator> navigator;
    };

} // namespace armarx::navigation::skills
