/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Reister ( fabian dot reister at kit dot edu )
 * @date       2023
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

#include <RobotAPI/libraries/skills/core/Skill.h>
#include <RobotAPI/libraries/skills/core/SkillDescription.h>
#include <RobotAPI/libraries/skills/core/SkillID.h>
#include <RobotAPI/libraries/skills/provider/SimpleSpecializedSkill.h>

#include <armarx/navigation/memory/client/graph/Reader.h>
#include <armarx/navigation/skills/aron/NavigateToNamedLocation.aron.generated.h>

namespace armarx::navigation::skills
{
    /**
     * @class NavigateToNamedLocation
     * @ingroup Library-armarx_navigation_skills
     *
     * @brief Resolve a named location and navigate to it.
     *
     * Subskills:
     * - armarx::navigation::skills::NavigateToLocation
     */
    class NavigateToNamedLocation :
        virtual public armarx::skills::SimpleSpecializedSkill<
            arondto::NavigateToNamedLocationParams>
    {
    public:
        using Params = arondto::NavigateToNamedLocationParams;
        using Base = ::armarx::skills::SimpleSpecializedSkill<Params>;

        static ::armarx::skills::SkillDescription GetSkillDescription();

        struct Services
        {
            armarx::navigation::memory::client::graph::Reader locationReader;
        };

        struct Properties
        {
            struct SubSkillIds
            {
                SubSkillIds();

                armarx::skills::SkillID navigateToLocation;
            };

            SubSkillIds subSkillIDs;
        };

        NavigateToNamedLocation(const Properties& properties, const Services& services);

    private:
        // SpecializedSkill interface
        // Enable each function you want to override.
        // bool isAvailable(const SpecializedInitInput&) const override;
        // ::armarx::skills::Skill::InitResult init(const SpecializedInitInput&) override;
        ::armarx::skills::Skill::MainResult main(const SpecializedMainInput& in) override;
        // ::armarx::skills::Skill::ExitResult exit(const SpecializedExitInput&) override;


    protected:
    private:
        Services services;
        Properties properties;
    };

} // namespace armarx::navigation::skills
