#include "RotateXDegrees.h"

#include <SimoxUtility/math/convert/deg_to_rad.h>

#include <armarx/navigation/skills/constants.h>

namespace armarx::navigation::skills
{

    RotateXDegrees::RotateXDegrees(const Services& srv) : Base(srv, DefaultSkillDescription())
    {
    }

    Eigen::Isometry3f
    RotateXDegrees::relativeTarget(const Base::SpecializedMainInput& in)
    {
        const float rotation = simox::math::deg_to_rad(in.parameters.rotationDegrees);
        return Eigen::Isometry3f(Eigen::AngleAxisf(rotation, Eigen::Vector3f::UnitZ()));
    }

    armarx::skills::SkillDescription
    RotateXDegrees::DefaultSkillDescription()
    {
        ParamType defaultParams;
        defaultParams.rotationDegrees = 0;

        return armarx::skills::SkillDescription{
            .skillId = {.skillName = constants::skill_names::RotateXDegrees},
            .description = "Rotate around the Z-axis by a specified degree value",
            .rootProfileDefaults = defaultParams.toAron(),
            .timeout = armarx::core::time::Duration::Minutes(1),
            .parametersType = ParamType::ToAronType()};
    }
} // namespace armarx::navigation::skills
