/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    navigation::NavigationCommands
 * @author     Fabian Reister ( fabian dot reister at kit dot edu )
 * @date       2022
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#include "CalculateRelativeMovement.h"

#include <Eigen/Core>
#include <Eigen/Geometry>

#include <RobotAPI/libraries/core/Pose.h>

namespace armarx::navigation::statecharts::navigation_commands
{
    // DO NOT EDIT NEXT LINE
    CalculateRelativeMovement::SubClassRegistry
        CalculateRelativeMovement::Registry(CalculateRelativeMovement::GetName(),
                                            &CalculateRelativeMovement::CreateInstance);

    void
    CalculateRelativeMovement::onEnter()
    {
        // put your user code for the enter-point here
        // execution time should be short (<100ms)
    }

    void
    CalculateRelativeMovement::run()
    {
        const float distance = in.getdistanceMillimeters();
        const Eigen::Vector3f movementDirection = in.getmovementDirection()->toEigen();

        Eigen::Vector3f relativeMovement = distance * movementDirection.normalized();

        const Eigen::Isometry3f robotRelativeMovement((Eigen::Translation3f(relativeMovement)));

        out.setrelativeMovement(toIce(robotRelativeMovement.matrix()));
        emitSuccess();
    }

    //void CalculateRelativeMovement::onBreak()
    //{
    //    // put your user code for the breaking point here
    //    // execution time should be short (<100ms)
    //}

    void
    CalculateRelativeMovement::onExit()
    {
        // put your user code for the exit point here
        // execution time should be short (<100ms)
    }

    // DO NOT EDIT NEXT FUNCTION
    XMLStateFactoryBasePtr
    CalculateRelativeMovement::CreateInstance(XMLStateConstructorParams stateData)
    {
        return XMLStateFactoryBasePtr(new CalculateRelativeMovement(stateData));
    }
} // namespace armarx::navigation::statecharts::navigation_commands
