/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    navigation::NavigationCommands
 * @author     armar-user ( armar6 at kit )
 * @date       2022
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#include "CalculateRotation.h"

#include <Eigen/Core>
#include <Eigen/Geometry>

#include <SimoxUtility/math/convert/deg_to_rad.h>

#include <RobotAPI/libraries/core/Pose.h>

namespace armarx::navigation::statecharts::navigation_commands
{
    // DO NOT EDIT NEXT LINE
    CalculateRotation::SubClassRegistry
        CalculateRotation::Registry(CalculateRotation::GetName(),
                                    &CalculateRotation::CreateInstance);

    void
    CalculateRotation::onEnter()
    {
        // put your user code for the enter-point here
        // execution time should be short (<100ms)
    }

    void
    CalculateRotation::run()
    {
        const float rotation = simox::math::deg_to_rad(in.getrotationDegrees());

        const Eigen::Isometry3f robotRelativeMovement(
            (Eigen::AngleAxisf(rotation, Eigen::Vector3f::UnitZ())));

        out.setrelativePose(toIce(robotRelativeMovement.matrix()));
        emitSuccess();
    }

    //void CalculateRotation::onBreak()
    //{
    //    // put your user code for the breaking point here
    //    // execution time should be short (<100ms)
    //}

    void
    CalculateRotation::onExit()
    {
        // put your user code for the exit point here
        // execution time should be short (<100ms)
    }

    // DO NOT EDIT NEXT FUNCTION
    XMLStateFactoryBasePtr
    CalculateRotation::CreateInstance(XMLStateConstructorParams stateData)
    {
        return XMLStateFactoryBasePtr(new CalculateRotation(stateData));
    }
} // namespace armarx::navigation::statecharts::navigation_commands
