/**
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Reister ( fabian dot reister at kit dot edu )
 * @date       2023
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once


#include <vector>

#include <boost/geometry.hpp>
#include <boost/geometry/geometries/point_xy.hpp>
#include <boost/geometry/geometries/polygon.hpp>

#include <Eigen/Core>

namespace armarx::navigation::util::geometry
{
    using point_type = boost::geometry::model::d2::point_xy<float>;
    using polygon_type = boost::geometry::model::polygon<point_type>;

    polygon_type toPolygon(const std::vector<Eigen::Vector2f>& hull);
    point_type toPoint(const Eigen::Vector2f& pt);
    Eigen::Vector2f fromPoint(const point_type& pt);
    float computeDistance(const std::vector<polygon_type>& obstacles, const Eigen::Vector2f& pt);


} // namespace armarx::navigation::util::geometry
